@@ -2,7 +2,7 @@
  * CT-API library for the REINER SCT cyberJack pinpad/e-com USB.
  * Copyright (C) 2004  REINER SCT
  * Author: Harald Welte
- * Support: linux-usb@sii.li
+ * Support: support@reiner-sct.com
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,6 +27,7 @@
 #include <unistd.h>
 #include <stdarg.h>
 #include <time.h>
+#include <string.h>
 #include <usb.h>
 
 #include "ecom/ctapi-ecom.h"
@@ -34,6 +35,7 @@
 #include "ppa/cjppa.h"
 
 #include "ctn_list.h"
+#include "cjctapi_beep.h"
 
 #define CJPPA_USB_VENDOR_ID	0x0c4b
 
@@ -41,11 +43,13 @@
 #define CJPPA_USB_DEVICE_ID	0x0300
 
 #define cjctapi_enter() cjctapi_log("ENTERING\n")
-static FILE *__cjctapi_logfile = NULL;
+
+static struct beep_struct *beepstruct = NULL;
 
 #ifdef DEBUG_CJCTAPI
 #define cjctapi_log(format, args...) \
 	__cjctapi_log(__FILE__, __LINE__, __FUNCTION__, format, ## args)
+static FILE *__cjctapi_logfile = NULL;
 void __cjctapi_log(char *file, int line, const char *function, 
 		   const char *format, ...)
 {
@@ -74,6 +78,7 @@ void __cjctapi_log(char *file, int line, const char *function,
 
 static int usb_read = 0;
 
+/* find an apropriate USB device */
 static struct usb_device *find_rsct_usbdev(int num)
 {
 	struct usb_bus *busses, *bus;
@@ -107,12 +112,44 @@ static struct usb_device *find_rsct_usbdev(int num)
 }
 
 
+/* callback functions for ppa */
+static void CJPP_CALLBACK_TYPE cjppCallbackKey(CCID_CTX ctx, 
+						unsigned char status)
+{
+	u_int16_t pn = (u_int16_t) ctx;
+
+	cjctapi_log(stderr, "%s called for port %u\n", __FUNCTION__, pn);
+	if (beepstruct)
+		beep_whatever(beepstruct);
+}
+
+#if 0
+static void CJPP_CALLBACK_TYPE cjppCallbackStatus(CCID_CTX ctx, 
+						   unsigned char status)
+{
+	u_int16_t pn = (u_int16_t) ctx;
+
+	fprintf(stderr, "%s called for port %u\n", __FUNCTION__, pn);
+}
+#endif
+
+/* callback function for e-com */
+static void cjecomCallbackKey(struct cj_info *dev)
+{
+	u_int16_t pn = ctn_list_lookup_bydev(dev);
+
+	cjctapi_log(stderr, "%s called for port %u\n", __FUNCTION__, pn);
+	if (beepstruct)
+		beep_whatever(beepstruct);
+}
+
 IS8 CT_init(IU16 ctn, IU16 pn)
 {
 	int ret = CT_API_RV_OK;
-	int type;
+	int type = -1;
 	void *dev;
 	struct usb_device *udev;
+	struct usb_device fake_ecom_udev;
 	char filename[PATH_MAX+1];
 
 	cjctapi_enter();
@@ -122,18 +159,37 @@ IS8 CT_init(IU16 ctn, IU16 pn)
 		return CT_API_RV_ERR_INVALID;
 	}
 
-	udev = find_rsct_usbdev(pn);
-	if (!udev) {
-		cjctapi_log("no matching usb device found\n");
-		return CT_API_RV_ERR_INVALID;
+	if (pn & 0x8000) {
+		/* Special case for direct opening of ttyUSB* by libchipcard */
+		cjctapi_log("Using fake_ecom_udev ttyUSB hack\n");
+		pn = pn & ~0x8000;
+		memset(&fake_ecom_udev, 0, sizeof(fake_ecom_udev));
+		fake_ecom_udev.next = fake_ecom_udev.prev = &fake_ecom_udev;
+		fake_ecom_udev.descriptor.idVendor = CJPPA_USB_VENDOR_ID;
+		fake_ecom_udev.descriptor.idProduct = CJECOM_USB_DEVICE_ID;
+		strncpy(fake_ecom_udev.filename, "/dev/fake_ecom_udev",
+			PATH_MAX);
+		udev = &fake_ecom_udev;
+	} else {
+		/* Normal case: iterate over existing usb devices */
+		udev = find_rsct_usbdev(pn);
+		if (!udev) {
+			cjctapi_log("no matching usb device found\n");
+			return CT_API_RV_ERR_INVALID;
+		}
 	}
 
+	/* only initialize beep_struct once */
+	if (!beepstruct && !getenv("CJCTAPI_NO_KEYBEEP"))
+		beepstruct = beep_init();
+
 	switch (udev->descriptor.idProduct) {
 		case CJPPA_USB_DEVICE_ID:
 			cjctapi_log("detected pinpad_a at %s\n", 
 				    udev->filename);
 			type = CJ_TYPE_PPA;
-			dev = ctapiInit(udev->filename, NULL, NULL, NULL);
+			dev = ctapiInit(udev->filename, (void *)pn,
+					NULL, &cjppCallbackKey);
 			cjctapi_log("ctapiInit returned %p\n", dev);
 			if (!dev)
 				ret = CT_API_RV_ERR_HOST;
@@ -146,6 +202,11 @@ IS8 CT_init(IU16 ctn, IU16 pn)
 			ret = cjecom_CT_init(filename,
 					     (struct cj_info **)&dev);
 			cjctapi_log("cjecom_CT_init returned %d\n", ret);
+			if (!dev)
+				cjctapi_log("no device returned by "
+					     "cjecom_CT_init\n");
+			else
+				ret = cjecom_CT_keycb(dev, &cjecomCallbackKey);
 			break;
 		default:
 			cjctapi_log("unknown Device ID 0x%x found\n",
@@ -175,9 +236,6 @@ IS8 CT_data(IU16 ctn, IU8 *dad, IU8 *sad, IU16 lenc, IU8 *command,
 	void *dev;
 	int type = ctn_list_lookup(ctn, &dev);
 
-	if (type < 0)
-		return CT_API_RV_ERR_INVALID;
-
 	switch (type) {
 	case CJ_TYPE_ECOMPP:
 		ret = cjecom_CT_data(dev, dad, sad, lenc, command, lenr,
@@ -186,6 +244,9 @@ IS8 CT_data(IU16 ctn, IU8 *dad, IU8 *sad, IU16 lenc, IU8 *command,
 	case CJ_TYPE_PPA:
 		ret = ctapiData(dev, dad, sad, lenc, command, lenr, response);
 		break;
+	default:
+		ret = CT_API_RV_ERR_INVALID;
+		break;
 	}
 
 	return ret;
@@ -213,21 +274,33 @@ IS8 CT_close(IU16 ctn)
 	case CJ_TYPE_PPA:
 		ret = ctapiClose(dev);
 		break;
+	default:
+		ret = CT_API_RV_ERR_INVALID;
+		break;
 	}
 	ctn_list_del(ctn);
+
+	if (ctn_list_empty() && beepstruct) {
+		beep_fini(beepstruct);
+		beepstruct = NULL;
+	}
+
 	return ret;
 }
 
 /* Proprietary extension */
-IS8 CT_keycb( IU16 ctn, void (* cb)(void) )
+IS8 CT_keycb( IU16 ctn, void (* cb)(IU16 ctn, IU8 status) )
 {
-	int ret;
 	void *dev;
 	int type = ctn_list_lookup(ctn, &dev);
 
 	if (type < 0)
 		return CT_API_RV_ERR_INVALID;
 
+	if (ctn_list_set_keycb(ctn, cb) < 0)
+		return CT_API_RV_ERR_INVALID;
+
+#if 0
 	switch (type) {
 	case CJ_TYPE_ECOMPP:
 		ret = cjecom_CT_keycb(dev, cb);
@@ -238,5 +311,6 @@ IS8 CT_keycb( IU16 ctn, void (* cb)(void) )
 		break;
 	}
 
-	return ret;
+#endif
+	return CT_API_RV_OK;
 }
