@@ -6,9 +6,9 @@
 
 #if defined(_LINUX) && defined(CJPPA_DEBUG)
 #include <stdio.h>
-#define DEBUGP(x, ...) fprintf(stderr, "%s:%d:%s: " x, 			\
+#define DEBUGP(x, ...) fprintf(stderr, "%s:%d:%s: " x, 		\
 				   __FILE__, __LINE__, __FUNCTION__, 	\
-				   __VA_ARGS__)
+				   ## __VA_ARGS__)
 #else 
 #define DEBUGP(x, ...)
 #endif
